/*
 * Copyright (c) 2012 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*jslint devel: true*/
/*global window, document, tizen*/

(function start() {
    'use strict';

    /**
     * Canvas element.
     *
     * @private
     * @type {HTMLCanvasElement}
     */
    var canvas = null,

        /**
         * Canvas element context.
         *
         * @private
         * @type {CanvasRenderingContext2D}
         */
        context = null,

        /**
         * Content element.
         *
         * @private
         * @type {HTMLElement}
         */
        content = null,

        /**
         * Object that stores information about Touch objects
         * for all the fingers that touch the screen.
         *
         * @private
         * @type {object}
         */
        drawPath = {},

        /**
         * Line color.
         *
         * @private
         * @type {string}
         */
        strokeColor = '',

        /**
         * Line width.
         *
         * @private
         * @type {number}
         */
        strokeWidth = 5;

    /**
     * Handles touchstart event on canvas element.
     *
     * @private
     * @param {TouchEvent} e
     */
    function touchStartHandler(e) {
        var touch = e.changedTouches[0];

        drawPath[touch.identifier] = touch;

        context.fillStyle = strokeColor;
        context.beginPath();
        context.arc(
            drawPath[touch.identifier].pageX - content.offsetLeft,
            drawPath[touch.identifier].pageY - content.offsetTop,
            strokeWidth / 2,
            0,
            Math.PI * 2,
            true
        );
        context.closePath();
        context.fill();
    }

    /**
     * Handles touchmove event on canvas element.
     *
     * @private
     * @param {TouchEvent} e
     */
    function touchMoveHandler(e) {
        var touches = e.changedTouches,
            touchesLength = touches.length,
            currentDrawPath = null,
            i = 0;

        context.lineWidth = strokeWidth;
        context.strokeStyle = strokeColor;
        context.lineJoin = 'round';

        for (i = 0; i < touchesLength; i += 1) {
            currentDrawPath = drawPath[touches[i].identifier];
            if (currentDrawPath !== undefined) {
                context.beginPath();
                context.moveTo(
                    currentDrawPath.pageX - content.offsetLeft,
                    currentDrawPath.pageY - content.offsetTop
                );
                context.lineTo(
                    touches[i].pageX - content.offsetLeft,
                    touches[i].pageY - content.offsetTop
                );
                context.closePath();
                context.stroke();

                drawPath[touches[i].identifier] = touches[i];
            }
        }
        e.preventDefault();
    }

    /**
     * Handles touchend event on canvas element.
     *
     * @private
     * @param {TouchEvent} e
     */
    function touchEndHandler(e) {
        drawPath[e.changedTouches[0].identifier] = undefined;
    }

    /**
     * Handles change event on line color selector element.
     *
     * @private
     * @param {Event} e
     */
    function changeStrokeColor(e) {
        strokeColor = e.target.value;
    }

    /**
     * Handles change event on line width selector element.
     *
     * @private
     * @param {Event} e
     */
    function changeStrokeWidth(e) {
        strokeWidth = e.target.value;
    }

    /**
     * Handles click event on clear button element.
     *
     * @private
     */
    function clearCanvas() {
        context.clearRect(0, 0, canvas.width, canvas.height);
    }

    /**
     * Initializes application.
     *
     * @public
     */
    function init() {
        var strokeColorSel = document.querySelector('.color-input'),
            strokeWidthSel = document.querySelector('.range-input'),
            clearBtn = document.querySelector('.clear-input');

        content = document.querySelector('.content');
        canvas = document.querySelector('.canvas');
        context = canvas.getContext('2d');

        // Canvas size setting
        canvas.width = content.clientWidth;
        canvas.height = content.clientHeight;

        // Touch events handler
        canvas.addEventListener('touchstart', touchStartHandler, false);
        canvas.addEventListener('touchend', touchEndHandler, false);
        canvas.addEventListener('touchmove', touchMoveHandler, false);

        // Apply eventHandler
        strokeColorSel.addEventListener('change', changeStrokeColor, false);
        strokeWidthSel.addEventListener('change', changeStrokeWidth, false);
        clearBtn.addEventListener('click', clearCanvas, false);

        // Add eventListener for tizenhwkey
        window.addEventListener('tizenhwkey', function onTizenHwKey(e) {
            if (e.keyName === 'back') {
                try {
                    tizen.application.getCurrentApplication().exit();
                } catch (err) {
                    console.error('Error: ', err);
                }
            }
        });
    }

    document.addEventListener('DOMContentLoaded', function onDocumentReady() {
        init();
    });
}());
